#define MODULE
#define __KERNEL__

/* Saint Michael, Linux Kernel Module.
 * Verions: 0.03
 *
 * June 1, 2001
 *
 *
 *    Copyright (C) 2001  Timothy Lalwess (lawless@netdoor.com)
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be
 *    useful, but WITHOUT ANY WARRANTY; without even the implied
 *    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *    PURPOSE.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 *    USA.
 *      
 *
 * 
 */

#include <linux/sys.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <asm/segment.h>
#include <asm/unistd.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/signal.h>
#include <linux/slab.h>
#include <asm/unistd.h>
#include <asm/current.h>
#include <sys/syscall.h>
#include <asm/errno.h>
#include <asm/ptrace.h>
#include <asm/pgtable.h>
#include "StMichael_lkm.h"

SM_SYSCALL_RECORD recorded_sys_call_table[NR_syscalls];

int
init_module(void)
{
    struct task_struct *p;
    struct task_struct *inittask;
    struct sm_argv_memory *memory;

#ifdef CLOAK
    struct module *m, **v;
#endif

     
    int             i;

    /*
     * Our memory is not cleared for us. We need to init our hashes so that
     * each hash is like null. 
     */

#ifndef SILENT
    printk("Loading %s ... ", VERSION);
#endif

#ifdef __SMP__

    rwlock_init(&sm_running);
#endif

    (void *) orig_init_module = sys_call_table[__NR_init_module]; 
    (void *) orig_delete_module = sys_call_table[__NR_delete_module];

    sys_call_table[__NR_init_module] = (void *) sm_init_module;
    sys_call_table[__NR_delete_module] = (void *) sm_delete_module; 

/* Now to Record the State of the Systemcall Table */

    for ( i = 0; i < NR_syscalls; i++ ) 
      {
#ifdef USE_MD5
        md5_state_t state;
        
        recorded_sys_call_table[i].orig_syscall = sys_call_table[i];

        md5_init(&state);
        md5_append(&state, (const md5_byte_t *) sys_call_table[i], 
                           md5_sample_len);
        md5_finish(&state, recorded_sys_call_table[i].recorded_digest);
#else

       recorded_sys_call_table[i] = sys_call_table[i];
#endif
      }


#ifdef CLOAK
      for ( m = THIS_MODULE; m->next != NULL; m = m->next );
      v = (struct module **) (++m);
      *v = (THIS_MODULE)->next;
#endif

#ifndef SILENT
    printk("Done. \n");
#endif

    return 0;
}

void
cleanup_module(void)
{

    sys_call_table[__NR_init_module] = (void *) orig_init_module;
    sys_call_table[__NR_delete_module] = (void *) orig_delete_module; 
     
     /* 
       We should wait untill all calls are finished before ripping the 
       syscalls out from under the world ;)
     */

#ifdef __SMP__
    write_lock(&sm_running);
    write_unlock(&sm_running);
#endif __SMP__
}
